@@ -17,9 +17,10 @@ use serde_json as json;
 use crate::v0;
 use crate::Location;
 
-pub(crate) use helm::Helm;
-use helper::{group_nodes_by_region, group_nodes_by_zone, is_aks, is_eks, NamesAndContexts};
-use kubeconfig::KubeconfigExt;
+pub(crate) use self::helm::Helm;
+
+use self::helper::{group_nodes_by_region, group_nodes_by_zone, is_aks, is_eks, NamesAndContexts};
+use self::kubeconfig::KubeconfigExt;
 
 mod crd;
 mod diagnose;
@@ -125,14 +126,44 @@ impl Kubectl {
         })
     }
 
+    pub(crate) fn default() -> anyhow::Result<Self> {
+        let kubeconfig = Kubeconfig::read()?;
+        let context = kubeconfig.current_context().unwrap_or_default().to_string();
+        let name = normalize_name(&context);
+
+        Ok(Self {
+            kubeconfig,
+            name,
+            context,
+        })
+    }
+
     pub(crate) fn context(&self) -> &str {
         &self.context
     }
 
+    pub(crate) fn all_contexts(&self) -> Vec<&str> {
+        self.kubeconfig.all_contexts()
+    }
+
     pub(crate) fn cluster_name(&self) -> &v0::ClusterName {
         &self.name
     }
 
+    pub(crate) fn set_cluster_name(self, name: v0::ClusterName) -> Self {
+        Self { name, ..self }
+    }
+
+    pub(crate) fn set_context(self, context: String) -> Self {
+        let name = normalize_name(&context);
+
+        Self {
+            name,
+            context,
+            ..self
+        }
+    }
+
     async fn get_client(&self) -> anyhow::Result<Client> {
         let context = Some(self.context.clone());
         let options = KubeConfigOptions {
