@@ -10,7 +10,7 @@ use std::str;
 
 use anyhow::Context;
 use console::{Style, Term};
-use dialoguer::{theme, Confirm, Input};
+use dialoguer::{theme, Confirm, Input, MultiSelect, Select};
 use futures::StreamExt;
 use itertools::Itertools;
 use serde::{de, Deserialize, Serialize};
@@ -683,51 +683,45 @@ impl Cli {
                 cleanup_grace,
                 azure,
             } => {
-                let interactive = !non_interactive;
-                let kubectls = k8s::Kubectl::with_names_and_contexts(names, contexts)?;
-
-                let no_default_storage_class = if no_state {
-                    true
-                } else {
-                    no_default_storage_class
-                };
-
-                let default_state = if no_default_storage_class {
-                    None
-                } else if default_storage_class.is_none() {
-                    states.first().map(ToString::to_string)
-                } else {
-                    default_storage_class
-                };
-
-                let states = if no_state { None } else { Some(states) };
-                let claim_unowned_states = !no_state_owner;
-
-                let helm = k8s::Helm::new(skip_helm);
-
-                azure.validate()?;
-
-                let register_cluster = RegisterCluster::new(
-                    helm,
-                    provider,
-                    namespace,
-                    states,
-                    default_state,
-                    claim_unowned_states,
-                    cleanup_grace,
-                    azure,
-                );
-                if interactive {
-                    // Right now interactive_register_cluster() does very much the same job as
-                    // non-interactive counterpart. In the future we may handle it differently.
+                let (kubectls, register_cluster) = if non_interactive {
                     statehub
-                        .interactive_register_cluster(kubectls, register_cluster)
-                        .await
+                        .non_interactive_register_cluster_helper(
+                            names,
+                            contexts,
+                            states,
+                            no_state,
+                            no_default_storage_class,
+                            default_storage_class,
+                            no_state_owner,
+                            namespace,
+                            skip_helm,
+                            provider,
+                            cleanup_grace,
+                            azure,
+                        )
+                        .await?
                 } else {
                     statehub
-                        .register_multiple_clusters(kubectls, register_cluster)
-                        .await
-                }
+                        .interactive_register_cluster_helper(
+                            names,
+                            contexts,
+                            states,
+                            no_state,
+                            no_default_storage_class,
+                            default_storage_class,
+                            no_state_owner,
+                            namespace,
+                            skip_helm,
+                            provider,
+                            cleanup_grace,
+                            azure,
+                        )
+                        .await?
+                };
+
+                statehub
+                    .register_multiple_clusters(kubectls, register_cluster)
+                    .await
             }
             Command::UnregisterCluster {
                 force,
@@ -989,15 +983,6 @@ impl StateHub {
             .print(&self.stdout, &self.style, self.json)
     }
 
-    async fn interactive_register_cluster(
-        &self,
-        kubectls: Vec<k8s::Kubectl>,
-        register_cluster: RegisterCluster,
-    ) -> anyhow::Result<()> {
-        self.register_multiple_clusters(kubectls, register_cluster)
-            .await
-    }
-
     async fn register_multiple_clusters(
         &self,
         kubectls: Vec<k8s::Kubectl>,
@@ -1096,7 +1081,7 @@ impl StateHub {
         force: bool,
         non_interactive: bool,
     ) -> anyhow::Result<()> {
-        if force || non_interactive || self.confirm("Are you sure?") {
+        if force || non_interactive || self.confirm_with_default("Are you sure?", false) {
             log::info!("Make sure all the pods using statehub are terminated");
             log::info!("Uninstall helm");
             for kubectl in kubectls {
@@ -1283,7 +1268,7 @@ impl StateHub {
         kubectls: Vec<k8s::Kubectl>,
         force: bool,
     ) -> anyhow::Result<()> {
-        if force || self.confirm("Are you sure?") {
+        if force || self.confirm_with_default("Are you sure?", false) {
             log::info!("Make sure all the pods using statehub are terminated");
             log::info!("Uninstall helm");
             for kubectl in kubectls {
@@ -1428,11 +1413,11 @@ impl StateHub {
         Ok(())
     }
 
-    fn confirm(&self, prompt: impl Into<String>) -> bool {
+    fn confirm_with_default(&self, prompt: impl Into<String>, yes: bool) -> bool {
         let theme = self.theme();
         Confirm::with_theme(theme)
             .with_prompt(prompt)
-            .default(false)
+            .default(yes)
             .show_default(true)
             .interact()
             .unwrap_or(false)
@@ -1472,6 +1457,53 @@ impl StateHub {
             .with_prompt(prompt)
             .interact_text_on(&self.stdout)
     }
+
+    fn input_with_default<T>(&self, prompt: impl Into<String>, default: T) -> io::Result<T>
+    where
+        T: Clone + str::FromStr + fmt::Display,
+        T::Err: fmt::Display + fmt::Debug,
+    {
+        Input::with_theme(self.theme())
+            .with_prompt(prompt)
+            .default(default)
+            .interact_text_on(&self.stdout)
+    }
+
+    fn select_with_default(
+        &self,
+        prompt: impl Into<String>,
+        items: Vec<&str>,
+        default: impl ToString,
+    ) -> io::Result<String> {
+        let items = items
+            .into_iter()
+            .map(ToString::to_string)
+            .collect::<Vec<_>>();
+        let default = default.to_string();
+        let default = items.iter().position(|item| item == &default).unwrap_or(0);
+        Select::with_theme(self.theme())
+            .with_prompt(prompt)
+            .items(&items)
+            .default(default)
+            .interact_on(&self.stdout)
+            .map(|idx| items[idx].clone())
+    }
+
+    fn multiselect_with_default<P, T>(&self, prompt: P, items: &[T]) -> io::Result<Vec<T>>
+    where
+        P: Into<String>,
+        T: Clone + ToString,
+    {
+        let items = MultiSelect::with_theme(self.theme())
+            .with_prompt(prompt)
+            .items(items)
+            .interact_on(&self.stdout)?
+            .into_iter()
+            .map(|idx| items[idx].clone())
+            .collect();
+
+        Ok(items)
+    }
 }
 
 impl str::FromStr for Entity {
