@@ -57,6 +57,168 @@ impl RegisterCluster {
 }
 
 impl StateHub {
+    #[allow(clippy::too_many_arguments)]
+    pub(super) async fn interactive_register_cluster_helper(
+        &self,
+        names: Vec<v0::ClusterName>,
+        contexts: Vec<String>,
+        states: Vec<v0::StateName>,
+        no_state: bool,
+        no_default_storage_class: bool,
+        default_storage_class: Option<String>,
+        no_state_owner: bool,
+        namespace: String,
+        skip_helm: bool,
+        provider: Option<v0::Provider>,
+        cleanup_grace: String,
+        azure: AzureCredentials,
+    ) -> anyhow::Result<(Vec<k8s::Kubectl>, RegisterCluster)> {
+        anyhow::ensure!(
+            !(names.len() > 1 && contexts.len() > 1),
+            "Multiple clusters and contexts specified - not supported for interactive register-cluster"
+        );
+
+        let kubectl = k8s::Kubectl::default()?;
+        let all_contexts = kubectl.all_contexts();
+        let default_context = contexts
+            .into_iter()
+            .next()
+            .filter(|ctx| all_contexts.contains(&ctx.as_str()));
+
+        let context = if let Some(context) = default_context {
+            context
+        } else {
+            self.select_with_default(
+                "Select KUBECONFIG context to register",
+                all_contexts,
+                kubectl.context(),
+            )
+            .unwrap_or_default()
+        };
+
+        let kubectl = kubectl.set_context(context);
+
+        let default_name = names
+            .into_iter()
+            .next()
+            .unwrap_or_else(|| kubectl.cluster_name().clone());
+
+        let cluster_name = self.input_with_default("Choose K8s cluster name", default_name)?;
+
+        let kubectl = kubectl.set_cluster_name(cluster_name);
+
+        let namespace =
+            self.input_with_default("Choose K8s Namespace to install statehub", namespace)?;
+
+        let skip_helm = skip_helm
+            || !self.confirm_with_default(
+                "Run 'helm install' to install Statehub K8s components",
+                true,
+            );
+        let helm = k8s::Helm::new(skip_helm);
+
+        let all_states = self.api.get_all_states().await.unwrap_or_default();
+        let unowned_states = all_states.iter().any(|state| state.owner.is_none());
+        let all_states = all_states
+            .into_iter()
+            .map(|state| state.name)
+            .collect::<Vec<_>>();
+
+        let no_default_storage_class = if no_state {
+            true
+        } else {
+            no_default_storage_class
+        };
+
+        let default_state = if no_default_storage_class {
+            None
+        } else if default_storage_class.is_none() {
+            states.first().map(ToString::to_string)
+        } else {
+            default_storage_class
+        };
+
+        let states = if no_state || all_states.is_empty() {
+            None
+        } else if states.is_empty() {
+            Some(states)
+        } else {
+            self.multiselect_with_default("Select states to use with this cluster", &all_states)
+                .ok()
+        };
+
+        let claim_unowned_states = !no_state_owner
+            && unowned_states
+            && self.confirm_with_default("Claim ownership of unowned states", true);
+
+        azure.validate()?;
+
+        let register_cluster = RegisterCluster::new(
+            helm,
+            provider,
+            namespace,
+            states,
+            default_state,
+            claim_unowned_states,
+            cleanup_grace,
+            azure,
+        );
+
+        Ok((vec![kubectl], register_cluster))
+    }
+
+    #[allow(clippy::too_many_arguments)]
+    pub(super) async fn non_interactive_register_cluster_helper(
+        &self,
+        names: Vec<v0::ClusterName>,
+        contexts: Vec<String>,
+        states: Vec<v0::StateName>,
+        no_state: bool,
+        no_default_storage_class: bool,
+        default_storage_class: Option<String>,
+        no_state_owner: bool,
+        namespace: String,
+        skip_helm: bool,
+        provider: Option<v0::Provider>,
+        cleanup_grace: String,
+        azure: AzureCredentials,
+    ) -> anyhow::Result<(Vec<k8s::Kubectl>, RegisterCluster)> {
+        let kubectls = k8s::Kubectl::with_names_and_contexts(names, contexts)?;
+        let no_default_storage_class = if no_state {
+            true
+        } else {
+            no_default_storage_class
+        };
+
+        let default_state = if no_default_storage_class {
+            None
+        } else if default_storage_class.is_none() {
+            states.first().map(ToString::to_string)
+        } else {
+            default_storage_class
+        };
+
+        let states = if no_state { None } else { Some(states) };
+        let claim_unowned_states = !no_state_owner;
+
+        let helm = k8s::Helm::new(skip_helm);
+
+        azure.validate()?;
+
+        let register_cluster = RegisterCluster::new(
+            helm,
+            provider,
+            namespace,
+            states,
+            default_state,
+            claim_unowned_states,
+            cleanup_grace,
+            azure,
+        );
+
+        Ok((kubectls, register_cluster))
+    }
+
     pub(super) async fn register_cluster_helper(
         &self,
         kubectl: k8s::Kubectl,
@@ -124,18 +286,21 @@ impl StateHub {
         let name = kubectl.cluster_name();
         let cluster = self.api.get_cluster(name).await?;
 
-        let (statehub_cluster_name, namespace) =
-            kubectl.get_statehub_cluster_name_and_namespace().await?;
-        anyhow::ensure!(
-            *name == statehub_cluster_name,
-            "Cluster name mismatch: K8s thinks it is '{}', while Statehub thinks it is  '{}'",
-            statehub_cluster_name,
-            name,
-        );
-        self.relinquish_states_helper(name).await?;
+        if let Ok((statehub_cluster_name, namespace)) =
+            kubectl.get_statehub_cluster_name_and_namespace().await
+        {
+            anyhow::ensure!(
+                *name == statehub_cluster_name,
+                "Cluster name mismatch: K8s thinks it is '{}', while Statehub thinks it is  '{}'",
+                statehub_cluster_name,
+                name,
+            );
 
-        self.helm_uninstall_helper(&kubectl, &cluster, &namespace, &helm)
-            .await?;
+            self.helm_uninstall_helper(&kubectl, &cluster, &namespace, &helm)
+                .await?;
+        }
+
+        self.relinquish_states_helper(name).await?;
 
         self.api
             .unregister_cluster(name)
